
#property copyright " 2008 BJF Trading Group"
#property link      "www.iticsoftware.com"

#define major   1
#define minor   0

extern string _tmp1_ = " --- Trade params ---";
extern double Lot1 = 1.0;
extern double Lot2 = 5.0;
extern double Lot3 = 4.0;
extern double Lot4 = 3.0;
extern double Lot5 = 2.0;
extern int StopLoss = 100;
extern int Slippage = 3;
extern int Magic = 20080918;

extern string _tmp2_ = " --- Trailing ---";
extern bool TrailingOn = false;
extern int TrailingStart = 30;
extern int TrailingSize = 30;

extern string _tmp3_ = " --- Chart ---";
extern color clBuy = DodgerBlue;
extern color clSell = Red;
extern color clModify = Silver;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include <stdlib.mqh>
#include <stderror.mqh>

int RepeatN = 3;
int BuyCnt, SellCnt;
int BuyStopCnt, SellStopCnt;

void init() 
{
}

void deinit() 
{
}

void start() 
{
  //-----
  
  if (TrailingOn) TrailPositions();  

  //-----
  
  double tp = 0.0;
  
  string obj_name = "Line6";
  if (ObjectFind(obj_name) != -1)
  {
    tp = ObjectGet(obj_name, OBJPROP_PRICE1);
  }

  ProcessLine1("Line1", Lot1, tp);
  ProcessLine1("Line2", Lot2, tp);
  ProcessLine1("Line3", Lot3, tp);
  ProcessLine1("Line4", Lot4, tp);
  ProcessLine1("Line5", Lot5, tp);        
}

void ProcessLine1(string line_name, double lot, double tp)
{
  string obj_name = line_name;
  if (ObjectFind(obj_name) == -1) return;

  double price = ObjectGet(obj_name, OBJPROP_PRICE1);
  int StopLevel = MarketInfo(Symbol(), MODE_STOPLEVEL) + 1;
  
  RecountOrdersEx(line_name);
  if (BuyCnt+SellCnt > 0 || BuyStopCnt+SellStopCnt > 0) return;

  int ticket;
  double sl;
  
  if (price > Ask + StopLevel*Point)
  {
    for (int i=0; i<RepeatN; i++)
    {
      sl = If(StopLoss > 0, price - StopLoss*Point, 0);

      ticket = SetBuyStop(Symbol(), lot, price, sl, tp, Magic, line_name);
      if (ticket > 0) break;
    }
  }

  if (price < Bid - StopLevel*Point)
  {
    for (i=0; i<RepeatN; i++)
    {
      sl = If(StopLoss > 0, price + StopLoss*Point, 0);

      ticket = SetSellStop(Symbol(), lot, price, sl, tp, Magic, line_name);
      if (ticket > 0) break;
    }
  }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

double If(bool cond, double if_true, double if_false)
{
  if (cond) return (if_true);
  return (if_false);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void RecountOrdersEx(string comment)
{
  BuyCnt = 0;
  SellCnt = 0;
  BuyStopCnt = 0;
  SellStopCnt = 0;

  int cnt = OrdersTotal();
  for (int i=0; i < cnt; i++) 
  {
    if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue;
    if (OrderSymbol() != Symbol()) continue;
    if (OrderMagicNumber() != Magic) continue;
    if (StringFind(OrderComment(), comment) == -1) continue;
    
    int type = OrderType();
    if (type == OP_BUY) BuyCnt++;
    if (type == OP_SELL) SellCnt++;
    if (type == OP_BUYSTOP) BuyStopCnt++;
    if (type == OP_SELLSTOP) SellStopCnt++;
  }
}

void TrailPositions()
{
  int cnt = OrdersTotal();
  for (int i=0; i<cnt; i++) 
  {
    if (!OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) continue;
    if (OrderSymbol() != Symbol()) continue;
    if (OrderMagicNumber() != Magic) continue;

    int type = OrderType();
    if (type == OP_BUY) 
    {
      if (Bid-OrderOpenPrice() > TrailingStart*Point) 
      {
        if (OrderStopLoss() < Bid - (TrailingSize+1)*Point) 
        {
          OrderModify(OrderTicket(), OrderOpenPrice(), Bid-TrailingSize*Point, OrderTakeProfit(), 0, clModify);
        }
      }
    }

    if (type == OP_SELL)
    {
      if (OrderOpenPrice()-Ask > TrailingStart*Point) 
      {
        if (OrderStopLoss() > Ask + (TrailingSize+1)*Point || OrderStopLoss() == 0) 
        {
          OrderModify(OrderTicket(), OrderOpenPrice(), Ask+TrailingSize*Point, OrderTakeProfit(), 0, clModify);
        }
      }
    }
  }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

int SleepOk = 2000;
int SleepErr = 6000;

int SetBuyStop(string symbol, double lot, double price, double sl, double tp, int magic, string comment="", datetime exp=0) 
{
  int dig = MarketInfo(symbol, MODE_DIGITS);

  price = NormalizeDouble(price, dig);
  sl = NormalizeDouble(sl, dig);
  tp = NormalizeDouble(tp, dig);
  
  string _lot = DoubleToStr(lot, 2);
  string _price = DoubleToStr(price, dig);
  string _sl = DoubleToStr(sl, dig);
  string _tp = DoubleToStr(tp, dig);

  Print("SetBuyStop \"", symbol, "\", ", _lot, ", ", _price, ", ", Slippage, ", ", _sl, ", ", _tp, ", ", magic, ", \"", comment, "\"");
  
  int res = OrderSend(symbol, OP_BUYSTOP, lot, price, Slippage, sl, tp, comment, magic, exp, clBuy);
  if (res >= 0) {
    Sleep(SleepOk);
    return (res);
  } 	
   	
  int code = GetLastError();
  Print("Error setting BUYSTOP order: ", ErrorDescription(code), " (", code, ")");
  Sleep(SleepErr);
	
  return (-1);
}

int SetSellStop(string symbol, double lot, double price, double sl, double tp, int magic, string comment="", datetime exp=0) 
{
  int dig = MarketInfo(symbol, MODE_DIGITS);

  price = NormalizeDouble(price, dig);
  sl = NormalizeDouble(sl, dig);
  tp = NormalizeDouble(tp, dig);
  
  string _lot = DoubleToStr(lot, 2);
  string _price = DoubleToStr(price, dig);
  string _sl = DoubleToStr(sl, dig);
  string _tp = DoubleToStr(tp, dig);

  Print("SetSellStop \"", symbol, "\", ", _lot, ", ", _price, ", ", Slippage, ", ", _sl, ", ", _tp, ", ", magic, ", \"", comment, "\"");
  
  int res = OrderSend(symbol, OP_SELLSTOP, lot, price, Slippage, sl, tp, comment, magic, exp, clSell);
  if (res >= 0) {
    Sleep(SleepOk);
    return (res);
  } 	
   	
  int code = GetLastError();
  Print("Error setting SELLSTOP order: ", ErrorDescription(code), " (", code, ")");
  Sleep(SleepErr);
	
  return (-1);
}